#include <allegro.h>
#include <alleggl.h>
#include <math.h>

#include "parody.h"
#include "resources.h"
#include "util.h"
#include "collision.h"
#include "input_map.h"
#include "stage.h"
#include "carriers.h"
#include "player.h"
#include "egg.h"
#include "message.h"

#define PLAYER_STEP 0.1
#define PLAYER_IMPULSE 0.55
#define PLAYER_GRAVITY 0.3
#define MAX_SHOOTING_TIME 5
#define STATS_FONT_SIZE 150
#define STATS_X -0.90
#define STATS_Y 0.95
#define EGG_STATS_X 0.1


enum {
    PLAYER_FRAME_STAND = 0,
    PLAYER_FRAME_JUMP,
    PLAYER_FRAME_FALL,
    PLAYER_FRAME_SHOOT,
    NUM_PLAYER_FRAMES,
};



struct PLAYER {
    float x, y;
    float vx, vy;
    float r;
    int is_walking;
    int is_jumping;
    int jump_time;
    int current_img;
    int is_shooting;
    int shoot_timer;
    GLuint img[NUM_PLAYER_FRAMES];
};



static PLAYER player;
static FONT *stats_font = NULL;



int init_player(DATAFILE *data) {
    BITMAP *bmp;

    player.x = 0;
    player.y = 0;
    player.vx = PLAYER_STEP;
    player.vy = 0;
    player.r = 0.15;
    player.is_walking = 0;
    player.is_jumping = 0;
    player.jump_time = 0;
    player.current_img = PLAYER_FRAME_STAND;
    player.is_shooting = 0;
    player.shoot_timer = MAX_SHOOTING_TIME;

    player.img[PLAYER_FRAME_STAND] = get_resource(DAT_PLAYER_STAND);
    player.img[PLAYER_FRAME_JUMP] = get_resource(DAT_PLAYER_JUMP);
    player.img[PLAYER_FRAME_FALL] = get_resource(DAT_PLAYER_FALL);
    player.img[PLAYER_FRAME_SHOOT] = get_resource(DAT_PLAYER_SHOOT);

    allegro_gl_set_texture_format(GL_ALPHA8);
    stats_font = allegro_gl_convert_allegro_font((FONT *) data[DAT_FONT_SMALL].dat,
	                                              AGL_FONT_TYPE_TEXTURED, STATS_FONT_SIZE);

    return 1;
}



int player_fell() {
    return player.y < -1.0;
}



static void move_horizontal(INPUT_MAP *map) {

    if (key[map->left]/* && !player.is_walking*/) {
        player.x -= player.vx;
        player.is_walking = 1;
        if (player.x < -1.0) {
            player.x = -1.0;
        }
    }
    if (key[map->right]/* && !player.is_walking*/) {
        player.is_walking = 1;
        player.x += player.vx;
        if (player.x > 1.0) {
            player.x = 1.0;
        }
    }

    if (!key[map->right] && !key[map->left]) {
        player.is_walking = 0;
    }
}



static void move_vertical(INPUT_MAP *map) {

    COLLISION_DATA c;
    COLLISION_RESULT res;

    c.x = player.x;
    c.y = player.y;
    c.r = player.r;

    res = stage_get_collision(&c);

    if (res.collision && res.vertical == DOWN) {
        player.vy = res.d;
        player.is_jumping = 0;

        if (player.is_shooting) {
            if (player.shoot_timer > 0) {
                --player.shoot_timer;
            } else {
                player.shoot_timer = MAX_SHOOTING_TIME;
                player.is_shooting = 0;
            }
        } else {
            player.current_img = PLAYER_FRAME_STAND;
        }

        if (key[map->jump]) {
            player.jump_time = 0;
            player.is_jumping = 1;
            player.current_img = PLAYER_FRAME_JUMP;
        }
    } else {
        player.vy = -0.1;
    }

    if (player.is_jumping) {
        player.vy = PLAYER_IMPULSE - (PLAYER_GRAVITY * player.jump_time);
        ++player.jump_time;
    }

    if (player.vy < 0 && player.is_jumping) {
        player.current_img = PLAYER_FRAME_FALL;
    }

    if (player.vy < 0 && fabs(player.vy) > player.r) {
        player.vy = -1 * player.r;
    }

    player.y += player.vy;
}



static void get_eggs() {
    COLLISION_DATA c;
    COLLISION_RESULT res;

    c.x = player.x;
    c.y = player.y;
    c.r = player.r;

    res = carriers_get_collision(&c);

    if (res.collision) {
        add_egg();
    }
}



static void player_shoot(INPUT_MAP *map) {
    if (key[map->shoot] && !egg_is_shooting()) {
        if (egg_shoot(player.x, player.y)) {
            player.current_img = PLAYER_FRAME_SHOOT;
            player.is_shooting = 1;
            set_message(player.x + player.r, player.y, "NNNG!");
        }
    }
}



#define MAX_FRAME_TIME 3
static int frame_timer = MAX_FRAME_TIME;



void update_player() {
    INPUT_MAP *map = get_input_map();

    if (frame_timer > 0) {
        --frame_timer;
    } else {
        frame_timer = MAX_FRAME_TIME;
        move_vertical(map);
        move_horizontal(map);
        get_eggs();
        player_shoot(map);
    }
}



void draw_player() {

    glColor3f(1.0, 1.0, 1.0);
    glEnable(GL_TEXTURE_2D);

    glEnable(GL_ALPHA_TEST);
    glBindTexture(GL_TEXTURE_2D, player.img[player.current_img]);
    glBegin(GL_QUADS);
        glTexCoord2i(0, 1);		//Define texture coordinates
        glVertex2f(player.x - player.r, player.y + player.r);
        glTexCoord2i(0, 0);
        glVertex2f(player.x - player.r, player.y - player.r);
        glTexCoord2i(1, 0);
        glVertex2f(player.x + player.r, player.y - player.r);
        glTexCoord2i(1, 1);
        glVertex2f(player.x + player.r, player.y + player.r);
    glEnd();
	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable(GL_ALPHA_TEST);
/*
    glBegin(GL_LINE_LOOP);
    glColor3f(1.0, 1.0, 1.0);
    my_draw_circle(player.x, player.y, player.r);
    glEnd();
*/
}



void draw_player_stats() {
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);

    allegro_gl_printf(stats_font, STATS_X, STATS_Y, 0, makecol(128,255,128), "LEVEL:");
    allegro_gl_printf(stats_font, STATS_X + 0.4, STATS_Y, 0, makecol(255,255,255), "%d", troll_get_difficulty() + 1);

    allegro_gl_printf(stats_font, EGG_STATS_X, STATS_Y, 0, makecol(128,255,128), "EGGS:");
    glDisable(GL_BLEND);
}



void end_player() {
    if (stats_font) {
        allegro_gl_destroy_font(stats_font);
    }
}
